from hatak.plugin import RequestPlugin


class AddJsRequestPlugin(RequestPlugin):

    def __init__(self):
        super().__init__('add_js')

    def __call__(self, code):
        if code not in self.registry['js_codes']:
            self.registry['js_codes'].append(code)
        return ''


class AddCssLinkRequestPlugin(RequestPlugin):

    def __init__(self):
        super().__init__('add_css_link')

    def __call__(self, url):
        link = self.request.get_static(url)
        if link not in self.registry['css_links']:
            self.registry['css_links'].append(link)


class AddJsLinkRequestPlugin(RequestPlugin):

    def __init__(self):
        super().__init__('add_js_link')

    def __call__(self, url):
        link = self.request.get_static(url)
        if link not in self.registry['js_links']:
            self.registry['js_links'].append(link)


class GetStaticRequestPlugin(RequestPlugin):

    def __init__(self):
        super().__init__('get_static')

    def __call__(self, url):
        return self.request.static_path(self.settings['static'] + url)



from hatak.plugin import Plugin
from hatak.controller import ControllerPlugin
from haplugin.jinja2 import Jinja2Plugin

from .helper import StaticHelper
from .request_plugins import AddJsRequestPlugin, AddCssLinkRequestPlugin
from .request_plugins import AddJsLinkRequestPlugin, GetStaticRequestPlugin


class StaticPlugin(Plugin):

    def add_to_registry(self):
        self.registry['js_links'] = []
        self.registry['js_codes'] = []
        self.registry['css_links'] = []

    def add_request_plugins(self):
        self.add_request_plugin(AddJsRequestPlugin)
        self.add_request_plugin(AddCssLinkRequestPlugin)
        self.add_request_plugin(AddJsLinkRequestPlugin)
        self.add_request_plugin(GetStaticRequestPlugin)

    def add_controller_plugins(self):
        self.add_controller_plugin(StaticControllerPlugin)

    def add_depedency_plugins(self):
        self.app._validate_dependency_plugin(Jinja2Plugin)


class StaticControllerPlugin(ControllerPlugin):

    def make_helpers(self):
        self.add_helper(
            'static', StaticHelper)

        static = self.controller.data['static']
        for link in self.settings.get('css', []):
            static.add_css_link(link)

        for link in self.settings.get('js', []):
            static.add_js_link(link)



from haplugin.jinja2 import Jinja2HelperMany


class StaticHelper(Jinja2HelperMany):
    prefix = 'haplugin.statics:templates'

    def __init__(self, request):
        super().__init__(request)

    @property
    def js_links(self):
        return self.request.registry['js_links']

    @property
    def css_links(self):
        return self.request.registry['css_links']

    @property
    def js_codes(self):
        return self.request.registry['js_codes']

    def add_js_link(self, link):
        self.request.add_js_link(link)
        return ''

    def add_css_link(self, link):
        self.request.add_css_link(link)
        return ''

    def generate_js_links(self):
        return self.render_for('js_links.jinja2', {'links': self.js_links})

    def generate_css_links(self):
        return self.render_for('css_links.jinja2', {'links': self.css_links})

    def generate_js_codes(self):
        return self.render_for('js_codes.jinja2', {'codes': self.js_codes})



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
    'hatak>=0.2.1',
    'hatak_jinja2',
]

if __name__ == '__main__':
    setup(
        name='Hatak_Statics',
        version='0.2.1',
        description='Statics plugin for Hatak.',
        license='Apache License 2.0',
        packages=find_packages('src'),
        package_dir={'': 'src'},
        namespace_packages=['haplugin'],
        install_requires=install_requires,
        include_package_data=True,
        zip_safe=False,
        package_data={
            'haplugin': [
                'statics/templates/*.jinja2'
            ],
        }
    )
